# This file was auto-generated by Fern from our API Definition.

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.request_options import RequestOptions
from .raw_client import AsyncRawCallsClient, RawCallsClient
from .types.post_conversation_outbound_response import PostConversationOutboundResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class CallsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawCallsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawCallsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawCallsClient
        """
        return self._raw_client

    def start_an_outbound_call(
        self,
        *,
        agent_id: str,
        phone_number: str,
        variables: typing.Optional[typing.Dict[str, typing.Any]] = OMIT,
        from_product_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PostConversationOutboundResponse:
        """
        Initiates an outbound conversation with a specified agent and phone number.

        Parameters
        ----------
        agent_id : str
            The ID of the agent initiating the conversation

        phone_number : str
            The phone number to call

        variables : typing.Optional[typing.Dict[str, typing.Any]]
            The variables to pass to the conversation

        from_product_id : typing.Optional[str]
            The ID of the product to use for the call (get this from the /product/phone-numbers endpoint)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PostConversationOutboundResponse
            Successfully started the outbound conversation

        Examples
        --------
        from smallestai import SmallestAI

        client = SmallestAI(
            token="YOUR_TOKEN",
        )
        client.atoms.calls.start_an_outbound_call(
            agent_id="60d0fe4f5311236168a109ca",
            phone_number="+1234567890",
        )
        """
        _response = self._raw_client.start_an_outbound_call(
            agent_id=agent_id,
            phone_number=phone_number,
            variables=variables,
            from_product_id=from_product_id,
            request_options=request_options,
        )
        return _response.data


class AsyncCallsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawCallsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawCallsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawCallsClient
        """
        return self._raw_client

    async def start_an_outbound_call(
        self,
        *,
        agent_id: str,
        phone_number: str,
        variables: typing.Optional[typing.Dict[str, typing.Any]] = OMIT,
        from_product_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PostConversationOutboundResponse:
        """
        Initiates an outbound conversation with a specified agent and phone number.

        Parameters
        ----------
        agent_id : str
            The ID of the agent initiating the conversation

        phone_number : str
            The phone number to call

        variables : typing.Optional[typing.Dict[str, typing.Any]]
            The variables to pass to the conversation

        from_product_id : typing.Optional[str]
            The ID of the product to use for the call (get this from the /product/phone-numbers endpoint)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PostConversationOutboundResponse
            Successfully started the outbound conversation

        Examples
        --------
        import asyncio

        from smallestai import AsyncSmallestAI

        client = AsyncSmallestAI(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.atoms.calls.start_an_outbound_call(
                agent_id="60d0fe4f5311236168a109ca",
                phone_number="+1234567890",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.start_an_outbound_call(
            agent_id=agent_id,
            phone_number=phone_number,
            variables=variables,
            from_product_id=from_product_id,
            request_options=request_options,
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ....core.pydantic_utilities import IS_PYDANTIC_V2
from ....core.serialization import FieldMetadata
from ....core.unchecked_base_model import UncheckedBaseModel


class GetAgentTemplateResponseDataItem(UncheckedBaseModel):
    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The ID of the agent template
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The name of the agent template
    """

    description: typing.Optional[str] = pydantic.Field(default=None)
    """
    The description of the agent template
    """

    avatar_url: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="avatarUrl"),
        pydantic.Field(alias="avatarUrl", description="The avatar URL of the agent template"),
    ] = None
    reference_url: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="referenceUrl"),
        pydantic.Field(alias="referenceUrl", description="The docs url of the agent template"),
    ] = None
    category: typing.Optional[str] = pydantic.Field(default=None)
    """
    The category of the agent template
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2
from ...core.serialization import FieldMetadata
from ...core.unchecked_base_model import UncheckedBaseModel
from .agent_dto_synthesizer_voice_config import AgentDtoSynthesizerVoiceConfig


class AgentDtoSynthesizer(UncheckedBaseModel):
    """
    The synthesizer (TTS) configuration of the agent
    """

    voice_config: typing_extensions.Annotated[
        typing.Optional[AgentDtoSynthesizerVoiceConfig],
        FieldMetadata(alias="voiceConfig"),
        pydantic.Field(alias="voiceConfig", description="The voice configuration of the synthesizer"),
    ] = None
    speed: typing.Optional[float] = pydantic.Field(default=None)
    """
    The speed of the synthesizer
    """

    consistency: typing.Optional[float] = pydantic.Field(default=None)
    """
    The consistency of the synthesizer
    """

    similarity: typing.Optional[float] = pydantic.Field(default=None)
    """
    The similarity of the synthesizer
    """

    enhancement: typing.Optional[float] = pydantic.Field(default=None)
    """
    The enhancement of the synthesizer
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# coding: utf-8

"""
    Agent Management API

    API for managing agents, their templates, and call logs

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from smallestai.atoms.models.agent_id_workflow_get200_response_data_nodes_inner_data_variables import AgentIdWorkflowGet200ResponseDataNodesInnerDataVariables
from typing import Optional, Set
from typing_extensions import Self

class AgentIdWorkflowGet200ResponseDataNodesInnerData(BaseModel):
    """
    Node-specific data and configuration
    """ # noqa: E501
    label: Optional[StrictStr] = Field(default=None, description="Display label for the node")
    action: Optional[StrictStr] = Field(default=None, description="The action/prompt text for the node")
    is_start_node: Optional[StrictBool] = Field(default=None, description="Whether this is the starting node", alias="isStartNode")
    static_text: Optional[StrictBool] = Field(default=None, description="Whether the text is static", alias="staticText")
    knowledge_base: Optional[StrictStr] = Field(default=None, description="Knowledge base ID for the node", alias="knowledgeBase")
    use_global_knowledge_base: Optional[StrictBool] = Field(default=None, description="Whether to use global knowledge base", alias="useGlobalKnowledgeBase")
    is_disconnected: Optional[StrictBool] = Field(default=None, description="Whether the node is disconnected", alias="isDisconnected")
    type: Optional[StrictStr] = Field(default=None, description="Node type")
    variables: Optional[AgentIdWorkflowGet200ResponseDataNodesInnerDataVariables] = None
    has_error: Optional[StrictBool] = Field(default=None, alias="hasError")
    validation_errors: Optional[List[StrictStr]] = Field(default=None, alias="validationErrors")
    http_request: Optional[Dict[str, Any]] = Field(default=None, description="HTTP request configuration (for pre_call_api nodes)", alias="httpRequest")
    response_data: Optional[Dict[str, Any]] = Field(default=None, description="Response data configuration (for pre_call_api nodes)", alias="responseData")
    __properties: ClassVar[List[str]] = ["label", "action", "isStartNode", "staticText", "knowledgeBase", "useGlobalKnowledgeBase", "isDisconnected", "type", "variables", "hasError", "validationErrors", "httpRequest", "responseData"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of AgentIdWorkflowGet200ResponseDataNodesInnerData from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of variables
        if self.variables:
            _dict['variables'] = self.variables.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of AgentIdWorkflowGet200ResponseDataNodesInnerData from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "label": obj.get("label"),
            "action": obj.get("action"),
            "isStartNode": obj.get("isStartNode"),
            "staticText": obj.get("staticText"),
            "knowledgeBase": obj.get("knowledgeBase"),
            "useGlobalKnowledgeBase": obj.get("useGlobalKnowledgeBase"),
            "isDisconnected": obj.get("isDisconnected"),
            "type": obj.get("type"),
            "variables": AgentIdWorkflowGet200ResponseDataNodesInnerDataVariables.from_dict(obj["variables"]) if obj.get("variables") is not None else None,
            "hasError": obj.get("hasError"),
            "validationErrors": obj.get("validationErrors"),
            "httpRequest": obj.get("httpRequest"),
            "responseData": obj.get("responseData")
        })
        return _obj





"""
Call management and analytics.

Usage:
    from smallestai.atoms.helpers import Call

    call = Call()
    call.get_calls()
    call.get_call("id")
"""

import os
from typing import Any, Dict, List, Optional, Union

import requests

# Default API base URL
DEFAULT_BASE_URL = "https://api.smallest.ai/atoms/v1"


class Call:
    """
    Manager for call operations and analytics.

    Can be used standalone:
        call = Call()
        call.get_calls()
    """

    def __init__(
        self,
        base_url: Optional[str] = None,
        api_key: Optional[str] = None,
    ):
        """
        Initialize Call manager.

        Args:
            base_url: API base URL (default: api.smallest.ai)
            api_key: API key (default: SMALLEST_API_KEY env var)
        """
        self.base_url = base_url or os.environ.get("SMALLEST_BASE_URL", DEFAULT_BASE_URL)
        self.api_key = api_key or os.environ.get("SMALLEST_API_KEY", "")

    def _get_headers(self) -> Dict[str, str]:
        return {
            "Authorization": f"Bearer {self.api_key}",
            "Content-Type": "application/json",
        }

    # =========================================================================
    # Call Analytics
    # =========================================================================

    def get_call(self, call_id: str) -> Dict[str, Any]:
        """Get details for a single call."""
        url = f"{self.base_url}/conversation/{call_id}"
        response = requests.get(url, headers=self._get_headers())
        response.raise_for_status()
        return response.json()  # type: ignore[no-any-return]

    def get_calls(
        self,
        agent_id: Optional[str] = None,
        campaign_id: Optional[str] = None,
        page: int = 1,
        limit: int = 10,
        status: Optional[str] = None,
        call_type: Optional[str] = None,
        search: Optional[str] = None,
    ) -> Dict[str, Any]:
        """Get paginated list of calls."""
        url = f"{self.base_url}/conversation"
        params: Dict[str, Union[str, int]] = {"page": page, "limit": limit}
        if agent_id:
            params["agentIds"] = agent_id
        if campaign_id:
            params["campaignIds"] = campaign_id
        if status:
            params["statusFilter"] = status
        if call_type:
            params["callTypes"] = call_type
        if search:
            params["search"] = search

        response = requests.get(url, headers=self._get_headers(), params=params)
        response.raise_for_status()
        return response.json()  # type: ignore[no-any-return]

    def search_calls(self, call_ids: List[str]) -> Dict[str, Any]:
        """Batch search calls by ID."""
        url = f"{self.base_url}/conversation/search"
        response = requests.post(
            url,
            headers=self._get_headers(),
            json={"callIds": call_ids},
        )
        response.raise_for_status()
        return response.json()  # type: ignore[no-any-return]

    # =========================================================================
    # Post-Call Analytics Configuration
    # =========================================================================

    def get_post_call_config(self, agent_id: str) -> Dict[str, Any]:
        """Get post-call analytics config for an agent."""
        url = f"{self.base_url}/agent/{agent_id}/post-call-analytics"
        response = requests.get(url, headers=self._get_headers())
        response.raise_for_status()
        return response.json()  # type: ignore[no-any-return]

    def set_post_call_config(
        self,
        agent_id: str,
        summary_prompt: Optional[str] = None,
        disposition_metrics: Optional[List[Dict[str, Any]]] = None,
    ) -> Dict[str, Any]:
        """Set post-call analytics config for an agent."""
        url = f"{self.base_url}/agent/{agent_id}/post-call-analytics"
        payload: Dict[str, Any] = {}
        if summary_prompt is not None:
            payload["summaryPrompt"] = summary_prompt
        if disposition_metrics is not None:
            payload["dispositionMetrics"] = disposition_metrics

        response = requests.post(url, headers=self._get_headers(), json=payload)
        response.raise_for_status()
        return response.json()  # type: ignore[no-any-return]



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ... import core
from ...core.api_error import ApiError
from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.http_response import AsyncHttpResponse, HttpResponse
from ...core.request_options import RequestOptions
from ...core.unchecked_base_model import construct_type
from ..errors.bad_request_error import BadRequestError
from ..errors.internal_server_error import InternalServerError
from ..errors.unauthorized_error import UnauthorizedError
from .types.add_voice_to_model_response import AddVoiceToModelResponse
from .types.delete_voice_clone_response import DeleteVoiceCloneResponse
from .types.get_cloned_voices_response import GetClonedVoicesResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawVoiceCloningClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def add_voice_to_model(
        self, *, display_name: str, file: core.File, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[AddVoiceToModelResponse]:
        """
        Add your voice using the Waves API.

        Parameters
        ----------
        display_name : str
            Display name for the voice clone.

        file : core.File
            See core.File for more documentation

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[AddVoiceToModelResponse]
            Voice clone created successfully
        """
        _response = self._client_wrapper.httpx_client.request(
            "api/v1/lightning-large/add_voice",
            base_url=self._client_wrapper.get_environment().waves,
            method="POST",
            data={
                "displayName": display_name,
            },
            files={
                "file": file,
            },
            request_options=request_options,
            omit=OMIT,
            force_multipart=True,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AddVoiceToModelResponse,
                    construct_type(
                        type_=AddVoiceToModelResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_cloned_voices(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[GetClonedVoicesResponse]:
        """
        Retrieve your cloned voices.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetClonedVoicesResponse]
            Voices retrieved successfully.
        """
        _response = self._client_wrapper.httpx_client.request(
            "api/v1/lightning-large/get_cloned_voices",
            base_url=self._client_wrapper.get_environment().waves,
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetClonedVoicesResponse,
                    construct_type(
                        type_=GetClonedVoicesResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def delete_voice_clone(
        self, *, voice_id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[DeleteVoiceCloneResponse]:
        """
        Delete a cloned voice using the new Waves API.

        Parameters
        ----------
        voice_id : str
            The unique identifier of the voice clone to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[DeleteVoiceCloneResponse]
            Voice clone deleted successfully
        """
        _response = self._client_wrapper.httpx_client.request(
            "api/v1/lightning-large",
            base_url=self._client_wrapper.get_environment().waves,
            method="DELETE",
            json={
                "voiceId": voice_id,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DeleteVoiceCloneResponse,
                    construct_type(
                        type_=DeleteVoiceCloneResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawVoiceCloningClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def add_voice_to_model(
        self, *, display_name: str, file: core.File, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[AddVoiceToModelResponse]:
        """
        Add your voice using the Waves API.

        Parameters
        ----------
        display_name : str
            Display name for the voice clone.

        file : core.File
            See core.File for more documentation

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[AddVoiceToModelResponse]
            Voice clone created successfully
        """
        _response = await self._client_wrapper.httpx_client.request(
            "api/v1/lightning-large/add_voice",
            base_url=self._client_wrapper.get_environment().waves,
            method="POST",
            data={
                "displayName": display_name,
            },
            files={
                "file": file,
            },
            request_options=request_options,
            omit=OMIT,
            force_multipart=True,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AddVoiceToModelResponse,
                    construct_type(
                        type_=AddVoiceToModelResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_cloned_voices(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[GetClonedVoicesResponse]:
        """
        Retrieve your cloned voices.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetClonedVoicesResponse]
            Voices retrieved successfully.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "api/v1/lightning-large/get_cloned_voices",
            base_url=self._client_wrapper.get_environment().waves,
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetClonedVoicesResponse,
                    construct_type(
                        type_=GetClonedVoicesResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def delete_voice_clone(
        self, *, voice_id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[DeleteVoiceCloneResponse]:
        """
        Delete a cloned voice using the new Waves API.

        Parameters
        ----------
        voice_id : str
            The unique identifier of the voice clone to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[DeleteVoiceCloneResponse]
            Voice clone deleted successfully
        """
        _response = await self._client_wrapper.httpx_client.request(
            "api/v1/lightning-large",
            base_url=self._client_wrapper.get_environment().waves,
            method="DELETE",
            json={
                "voiceId": voice_id,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DeleteVoiceCloneResponse,
                    construct_type(
                        type_=DeleteVoiceCloneResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ... import core
from ...core.api_error import ApiError
from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.http_response import AsyncHttpResponse, HttpResponse
from ...core.jsonable_encoder import jsonable_encoder
from ...core.request_options import RequestOptions
from ...core.unchecked_base_model import construct_type
from ..errors.bad_request_error import BadRequestError
from ..errors.internal_server_error import InternalServerError
from ..errors.unauthorized_error import UnauthorizedError
from .types.delete_knowledgebase_id_response import DeleteKnowledgebaseIdResponse
from .types.delete_knowledgebase_knowledge_base_id_items_knowledge_base_item_id_response import (
    DeleteKnowledgebaseKnowledgeBaseIdItemsKnowledgeBaseItemIdResponse,
)
from .types.get_knowledgebase_id_items_response import GetKnowledgebaseIdItemsResponse
from .types.get_knowledgebase_id_response import GetKnowledgebaseIdResponse
from .types.get_knowledgebase_response import GetKnowledgebaseResponse
from .types.post_knowledgebase_id_items_upload_media_response import PostKnowledgebaseIdItemsUploadMediaResponse
from .types.post_knowledgebase_response import PostKnowledgebaseResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawKnowledgeBaseClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get_all_knowledge_bases(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[GetKnowledgebaseResponse]:
        """
        Get all knowledge bases

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetKnowledgebaseResponse]
            A list of knowledge bases
        """
        _response = self._client_wrapper.httpx_client.request(
            "knowledgebase",
            base_url=self._client_wrapper.get_environment().atoms,
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetKnowledgebaseResponse,
                    construct_type(
                        type_=GetKnowledgebaseResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def create_a_knowledge_base(
        self,
        *,
        name: str,
        description: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[PostKnowledgebaseResponse]:
        """
        Create a knowledge base

        Parameters
        ----------
        name : str

        description : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[PostKnowledgebaseResponse]
            Knowledge base created successfully
        """
        _response = self._client_wrapper.httpx_client.request(
            "knowledgebase",
            base_url=self._client_wrapper.get_environment().atoms,
            method="POST",
            json={
                "name": name,
                "description": description,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PostKnowledgebaseResponse,
                    construct_type(
                        type_=PostKnowledgebaseResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_a_knowledge_base(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[GetKnowledgebaseIdResponse]:
        """
        Get a knowledge base

        Parameters
        ----------
        id : str
            The ID of the knowledge base

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetKnowledgebaseIdResponse]
            A knowledge base
        """
        _response = self._client_wrapper.httpx_client.request(
            f"knowledgebase/{jsonable_encoder(id)}",
            base_url=self._client_wrapper.get_environment().atoms,
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetKnowledgebaseIdResponse,
                    construct_type(
                        type_=GetKnowledgebaseIdResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def delete_a_knowledge_base(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[DeleteKnowledgebaseIdResponse]:
        """
        Delete a knowledge base

        Parameters
        ----------
        id : str
            The ID of the knowledge base

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[DeleteKnowledgebaseIdResponse]
            Knowledge base deleted successfully
        """
        _response = self._client_wrapper.httpx_client.request(
            f"knowledgebase/{jsonable_encoder(id)}",
            base_url=self._client_wrapper.get_environment().atoms,
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DeleteKnowledgebaseIdResponse,
                    construct_type(
                        type_=DeleteKnowledgebaseIdResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_all_knowledge_base_items(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[GetKnowledgebaseIdItemsResponse]:
        """
        Get all knowledge base items

        Parameters
        ----------
        id : str
            The ID of the knowledge base

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetKnowledgebaseIdItemsResponse]
            A list of knowledge base items
        """
        _response = self._client_wrapper.httpx_client.request(
            f"knowledgebase/{jsonable_encoder(id)}/items",
            base_url=self._client_wrapper.get_environment().atoms,
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetKnowledgebaseIdItemsResponse,
                    construct_type(
                        type_=GetKnowledgebaseIdItemsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def delete_a_knowledge_base_item(
        self,
        knowledge_base_id: str,
        knowledge_base_item_id: str,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[DeleteKnowledgebaseKnowledgeBaseIdItemsKnowledgeBaseItemIdResponse]:
        """
        Delete a knowledge base item

        Parameters
        ----------
        knowledge_base_id : str
            The ID of the knowledge base

        knowledge_base_item_id : str
            The ID of the knowledge base item

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[DeleteKnowledgebaseKnowledgeBaseIdItemsKnowledgeBaseItemIdResponse]
            Knowledge base item deleted successfully
        """
        _response = self._client_wrapper.httpx_client.request(
            f"knowledgebase/{jsonable_encoder(knowledge_base_id)}/items/{jsonable_encoder(knowledge_base_item_id)}",
            base_url=self._client_wrapper.get_environment().atoms,
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DeleteKnowledgebaseKnowledgeBaseIdItemsKnowledgeBaseItemIdResponse,
                    construct_type(
                        type_=DeleteKnowledgebaseKnowledgeBaseIdItemsKnowledgeBaseItemIdResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def upload_a_pdf_file_to_a_knowledge_base(
        self, id: str, *, media: core.File, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[PostKnowledgebaseIdItemsUploadMediaResponse]:
        """
        Upload a PDF file to a knowledge base. Only PDF files are accepted.

        Parameters
        ----------
        id : str
            The ID of the knowledge base

        media : core.File
            See core.File for more documentation

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[PostKnowledgebaseIdItemsUploadMediaResponse]
            Knowledge base item created successfully
        """
        _response = self._client_wrapper.httpx_client.request(
            f"knowledgebase/{jsonable_encoder(id)}/items/upload-media",
            base_url=self._client_wrapper.get_environment().atoms,
            method="POST",
            data={},
            files={
                "media": media,
            },
            request_options=request_options,
            omit=OMIT,
            force_multipart=True,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PostKnowledgebaseIdItemsUploadMediaResponse,
                    construct_type(
                        type_=PostKnowledgebaseIdItemsUploadMediaResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawKnowledgeBaseClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get_all_knowledge_bases(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[GetKnowledgebaseResponse]:
        """
        Get all knowledge bases

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetKnowledgebaseResponse]
            A list of knowledge bases
        """
        _response = await self._client_wrapper.httpx_client.request(
            "knowledgebase",
            base_url=self._client_wrapper.get_environment().atoms,
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetKnowledgebaseResponse,
                    construct_type(
                        type_=GetKnowledgebaseResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def create_a_knowledge_base(
        self,
        *,
        name: str,
        description: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[PostKnowledgebaseResponse]:
        """
        Create a knowledge base

        Parameters
        ----------
        name : str

        description : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[PostKnowledgebaseResponse]
            Knowledge base created successfully
        """
        _response = await self._client_wrapper.httpx_client.request(
            "knowledgebase",
            base_url=self._client_wrapper.get_environment().atoms,
            method="POST",
            json={
                "name": name,
                "description": description,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PostKnowledgebaseResponse,
                    construct_type(
                        type_=PostKnowledgebaseResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_a_knowledge_base(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[GetKnowledgebaseIdResponse]:
        """
        Get a knowledge base

        Parameters
        ----------
        id : str
            The ID of the knowledge base

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetKnowledgebaseIdResponse]
            A knowledge base
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"knowledgebase/{jsonable_encoder(id)}",
            base_url=self._client_wrapper.get_environment().atoms,
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetKnowledgebaseIdResponse,
                    construct_type(
                        type_=GetKnowledgebaseIdResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def delete_a_knowledge_base(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[DeleteKnowledgebaseIdResponse]:
        """
        Delete a knowledge base

        Parameters
        ----------
        id : str
            The ID of the knowledge base

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[DeleteKnowledgebaseIdResponse]
            Knowledge base deleted successfully
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"knowledgebase/{jsonable_encoder(id)}",
            base_url=self._client_wrapper.get_environment().atoms,
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DeleteKnowledgebaseIdResponse,
                    construct_type(
                        type_=DeleteKnowledgebaseIdResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_all_knowledge_base_items(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[GetKnowledgebaseIdItemsResponse]:
        """
        Get all knowledge base items

        Parameters
        ----------
        id : str
            The ID of the knowledge base

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetKnowledgebaseIdItemsResponse]
            A list of knowledge base items
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"knowledgebase/{jsonable_encoder(id)}/items",
            base_url=self._client_wrapper.get_environment().atoms,
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetKnowledgebaseIdItemsResponse,
                    construct_type(
                        type_=GetKnowledgebaseIdItemsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def delete_a_knowledge_base_item(
        self,
        knowledge_base_id: str,
        knowledge_base_item_id: str,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[DeleteKnowledgebaseKnowledgeBaseIdItemsKnowledgeBaseItemIdResponse]:
        """
        Delete a knowledge base item

        Parameters
        ----------
        knowledge_base_id : str
            The ID of the knowledge base

        knowledge_base_item_id : str
            The ID of the knowledge base item

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[DeleteKnowledgebaseKnowledgeBaseIdItemsKnowledgeBaseItemIdResponse]
            Knowledge base item deleted successfully
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"knowledgebase/{jsonable_encoder(knowledge_base_id)}/items/{jsonable_encoder(knowledge_base_item_id)}",
            base_url=self._client_wrapper.get_environment().atoms,
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DeleteKnowledgebaseKnowledgeBaseIdItemsKnowledgeBaseItemIdResponse,
                    construct_type(
                        type_=DeleteKnowledgebaseKnowledgeBaseIdItemsKnowledgeBaseItemIdResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def upload_a_pdf_file_to_a_knowledge_base(
        self, id: str, *, media: core.File, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[PostKnowledgebaseIdItemsUploadMediaResponse]:
        """
        Upload a PDF file to a knowledge base. Only PDF files are accepted.

        Parameters
        ----------
        id : str
            The ID of the knowledge base

        media : core.File
            See core.File for more documentation

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[PostKnowledgebaseIdItemsUploadMediaResponse]
            Knowledge base item created successfully
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"knowledgebase/{jsonable_encoder(id)}/items/upload-media",
            base_url=self._client_wrapper.get_environment().atoms,
            method="POST",
            data={},
            files={
                "media": media,
            },
            request_options=request_options,
            omit=OMIT,
            force_multipart=True,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    PostKnowledgebaseIdItemsUploadMediaResponse,
                    construct_type(
                        type_=PostKnowledgebaseIdItemsUploadMediaResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2
from ...core.unchecked_base_model import UncheckedBaseModel
from .single_prompt_config_tools_item_request_body_schema_properties_value import (
    SinglePromptConfigToolsItemRequestBodySchemaPropertiesValue,
)
from .single_prompt_config_tools_item_request_body_schema_type import SinglePromptConfigToolsItemRequestBodySchemaType


class SinglePromptConfigToolsItemRequestBodySchema(UncheckedBaseModel):
    """
    Optional for api_call type. Schema defining the structure of the request body
    """

    type: SinglePromptConfigToolsItemRequestBodySchemaType
    properties: typing.Dict[str, SinglePromptConfigToolsItemRequestBodySchemaPropertiesValue]
    required: typing.List[str]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2
from ...core.unchecked_base_model import UncheckedBaseModel
from .lightningv2request_language import Lightningv2RequestLanguage
from .lightningv2request_output_format import Lightningv2RequestOutputFormat


class Lightningv2Request(UncheckedBaseModel):
    text: str = pydantic.Field()
    """
    The text to convert to speech.
    """

    voice_id: str = pydantic.Field()
    """
    The voice identifier to use for speech generation.
    """

    sample_rate: typing.Optional[int] = pydantic.Field(default=None)
    """
    The sample rate for the generated audio.
    """

    speed: typing.Optional[float] = pydantic.Field(default=None)
    """
    The speed of the generated speech.
    """

    consistency: typing.Optional[float] = pydantic.Field(default=None)
    """
    This parameter controls word repetition and skipping. Decrease it to prevent skipped words, and increase it to prevent repetition.
    """

    similarity: typing.Optional[float] = pydantic.Field(default=None)
    """
    This parameter controls the similarity between the generated speech and the reference audio. Increase it to make the speech more similar to the reference audio.
    """

    enhancement: typing.Optional[float] = pydantic.Field(default=None)
    """
    Enhances speech quality at the cost of increased latency.
    """

    language: typing.Optional[Lightningv2RequestLanguage] = pydantic.Field(default=None)
    """
    Determines how numbers are spelled out. If set to 'en', numbers will be read as individual digits in English. If set to 'hi', numbers will be read as individual digits in Hindi.
    """

    output_format: typing.Optional[Lightningv2RequestOutputFormat] = pydantic.Field(default=None)
    """
    The format of the output audio.
    """

    pronunciation_dicts: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    The IDs of the pronunciation dictionaries to use for speech generation.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
import typing_extensions
from ....core.pydantic_utilities import IS_PYDANTIC_V2
from ....core.serialization import FieldMetadata
from ....core.unchecked_base_model import UncheckedBaseModel
from .get_conversation_response_data_logs_item_agent_disposition_config_item import (
    GetConversationResponseDataLogsItemAgentDispositionConfigItem,
)
from .get_conversation_response_data_logs_item_status import GetConversationResponseDataLogsItemStatus
from .get_conversation_response_data_logs_item_type import GetConversationResponseDataLogsItemType


class GetConversationResponseDataLogsItem(UncheckedBaseModel):
    id: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="_id"),
        pydantic.Field(alias="_id", description="The database ID of the call log"),
    ] = None
    call_id: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="callId"),
        pydantic.Field(alias="callId", description="The unique call identifier"),
    ] = None
    status: typing.Optional[GetConversationResponseDataLogsItemStatus] = pydantic.Field(default=None)
    """
    The status of the call
    """

    duration: typing.Optional[float] = pydantic.Field(default=None)
    """
    The duration of the call in seconds
    """

    from_: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="from"),
        pydantic.Field(alias="from", description="The phone number the call was made from"),
    ] = None
    to: typing.Optional[str] = pydantic.Field(default=None)
    """
    The phone number the call was made to
    """

    type: typing.Optional[GetConversationResponseDataLogsItemType] = pydantic.Field(default=None)
    """
    The type of call
    """

    agent_id: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="agentId"),
        pydantic.Field(alias="agentId", description="The ID of the agent that handled the call"),
    ] = None
    agent_name: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="agentName"),
        pydantic.Field(alias="agentName", description="The name of the agent"),
    ] = None
    recording_url: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="recordingUrl"),
        pydantic.Field(alias="recordingUrl", description="URL to the call recording (if available)"),
    ] = None
    recording_dual_url: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="recordingDualUrl"),
        pydantic.Field(alias="recordingDualUrl", description="URL to the dual-channel call recording (if available)"),
    ] = None
    disconnection_reason: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="disconnectionReason"),
        pydantic.Field(alias="disconnectionReason", description="The reason the call was disconnected"),
    ] = None
    retry_count: typing_extensions.Annotated[
        typing.Optional[int],
        FieldMetadata(alias="retryCount"),
        pydantic.Field(alias="retryCount", description="Number of retry attempts for this call"),
    ] = None
    created_at: typing_extensions.Annotated[
        typing.Optional[dt.datetime],
        FieldMetadata(alias="createdAt"),
        pydantic.Field(alias="createdAt", description="When the call was created"),
    ] = None
    disposition_metrics: typing_extensions.Annotated[
        typing.Optional[typing.Dict[str, str]],
        FieldMetadata(alias="dispositionMetrics"),
        pydantic.Field(alias="dispositionMetrics", description="Custom disposition metrics for the call"),
    ] = None
    agent_disposition_config: typing_extensions.Annotated[
        typing.Optional[typing.List[GetConversationResponseDataLogsItemAgentDispositionConfigItem]],
        FieldMetadata(alias="agentDispositionConfig"),
        pydantic.Field(alias="agentDispositionConfig", description="Configuration for disposition metrics"),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



"""
Type definitions for LLM clients.

Common types used across different LLM providers.
"""

from dataclasses import dataclass, field
from typing import Any, Dict, List, Optional


@dataclass
class ToolCall:
    """Function/tool call from LLM."""

    id: str
    name: str
    arguments: str  # JSON string

    def to_dict(self) -> Dict[str, Any]:
        """Convert to dictionary format."""
        return {
            "id": self.id,
            "type": "function",
            "function": {"name": self.name, "arguments": self.arguments},
        }


@dataclass
class ChatChunk:
    """Single chunk from streaming response."""

    content: Optional[str] = None
    tool_calls: Optional[List[ToolCall]] = None
    finish_reason: Optional[str] = None


@dataclass
class ChatResponse:
    """Complete non-streaming response."""

    content: Optional[str] = None
    tool_calls: Optional[List[ToolCall]] = None
    usage: Optional[Dict[str, Any]] = None


@dataclass
class ToolResult:
    """Result from tool execution."""

    tool_call_id: str
    name: str
    content: str
    is_error: bool = False

    def to_message(self) -> Dict[str, Any]:
        """Convert to OpenAI message format."""
        return {
            "role": "tool",
            "tool_call_id": self.tool_call_id,
            "name": self.name,
            "content": self.content,
        }



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2
from ...core.serialization import FieldMetadata
from ...core.unchecked_base_model import UncheckedBaseModel
from .single_prompt_config_tools_item_headers_item import SinglePromptConfigToolsItemHeadersItem
from .single_prompt_config_tools_item_method import SinglePromptConfigToolsItemMethod
from .single_prompt_config_tools_item_query_params_item import SinglePromptConfigToolsItemQueryParamsItem
from .single_prompt_config_tools_item_request_body_schema import SinglePromptConfigToolsItemRequestBodySchema
from .single_prompt_config_tools_item_response_variables_item import SinglePromptConfigToolsItemResponseVariablesItem
from .single_prompt_config_tools_item_type import SinglePromptConfigToolsItemType
from .single_prompt_config_tools_item_variables_extraction_schema_item import (
    SinglePromptConfigToolsItemVariablesExtractionSchemaItem,
)
from .single_prompt_config_tools_item_variables_item import SinglePromptConfigToolsItemVariablesItem


class SinglePromptConfigToolsItem(UncheckedBaseModel):
    type: SinglePromptConfigToolsItemType = pydantic.Field()
    """
    The type of function/tool
    """

    name: str = pydantic.Field()
    """
    Unique name for the function
    """

    description: str = pydantic.Field()
    """
    Description of what the function does
    """

    url: typing.Optional[str] = pydantic.Field(default=None)
    """
    Required for api_call type. The URL to make the HTTP request to
    """

    method: typing.Optional[SinglePromptConfigToolsItemMethod] = pydantic.Field(default=None)
    """
    Required for api_call type. HTTP method to use
    """

    timeout: typing.Optional[int] = pydantic.Field(default=None)
    """
    Optional for api_call type. Request timeout in milliseconds (1000-30000)
    """

    headers: typing.Optional[typing.List[SinglePromptConfigToolsItemHeadersItem]] = pydantic.Field(default=None)
    """
    Optional for api_call type. HTTP headers to include in the request
    """

    query_params: typing_extensions.Annotated[
        typing.Optional[typing.List[SinglePromptConfigToolsItemQueryParamsItem]],
        FieldMetadata(alias="queryParams"),
        pydantic.Field(
            alias="queryParams", description="Optional for api_call type. Query parameters to include in the request"
        ),
    ] = None
    request_body_schema: typing_extensions.Annotated[
        typing.Optional[SinglePromptConfigToolsItemRequestBodySchema],
        FieldMetadata(alias="requestBodySchema"),
        pydantic.Field(
            alias="requestBodySchema",
            description="Optional for api_call type. Schema defining the structure of the request body",
        ),
    ] = None
    response_variables: typing_extensions.Annotated[
        typing.Optional[typing.List[SinglePromptConfigToolsItemResponseVariablesItem]],
        FieldMetadata(alias="responseVariables"),
        pydantic.Field(
            alias="responseVariables",
            description="Optional for api_call type. Variables to extract from the API response",
        ),
    ] = None
    transfer_number: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="transferNumber"),
        pydantic.Field(
            alias="transferNumber", description="Required for transfer_call type. Phone number to transfer the call to"
        ),
    ] = None
    variables: typing.Optional[typing.List[SinglePromptConfigToolsItemVariablesItem]] = pydantic.Field(default=None)
    """
    Optional for extract_dynamic_variables type. Legacy field, use variablesExtractionSchema instead
    """

    variables_extraction_schema: typing_extensions.Annotated[
        typing.Optional[typing.List[SinglePromptConfigToolsItemVariablesExtractionSchemaItem]],
        FieldMetadata(alias="variablesExtractionSchema"),
        pydantic.Field(
            alias="variablesExtractionSchema",
            description="Required for extract_dynamic_variables type. Schema defining variables to extract from conversation",
        ),
    ] = None
    knowledge_base_id: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="knowledgeBaseId"),
        pydantic.Field(
            alias="knowledgeBaseId",
            description="Required for knowledge_base_search type. ID of the knowledge base to search",
        ),
    ] = None
    filler_phrases: typing_extensions.Annotated[
        typing.Optional[typing.List[str]],
        FieldMetadata(alias="fillerPhrases"),
        pydantic.Field(
            alias="fillerPhrases", description="Optional for knowledge_base_search type. Phrases to use while searching"
        ),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



"""
Base client interface for LLM providers.

Defines the common interface that all LLM clients should implement.
"""

from abc import ABC, abstractmethod
from typing import Any, AsyncIterator, Dict, List, Optional, Union

from smallestai.atoms.agent.clients.types import ChatChunk, ChatResponse


class BaseLLMClient(ABC):
    """Base interface for LLM clients."""

    @abstractmethod
    async def chat(
        self,
        messages: List[Dict[str, Any]],
        stream: bool = False,
        tools: Optional[List[Dict[str, Any]]] = None,
        **kwargs: Any,
    ) -> Union[ChatResponse, AsyncIterator[ChatChunk]]:
        """
        Make a chat completion request.

        Args:
            messages: Chat messages in OpenAI format
            stream: Whether to stream response
            tools: Tool/function definitions
            **kwargs: Provider-specific parameters

        Returns:
            ChatResponse for non-streaming, AsyncIterator[ChatChunk] for streaming
        """
        pass

    @abstractmethod
    async def aclose(self):
        """Close client and cleanup resources."""
        pass



import asyncio

import typer
from rich.console import Console
from rich.prompt import Prompt

from smallestai.cli.lib.atoms import AtomsAPIClient
from smallestai.cli.lib.auth import AuthClient

console = Console()


def initialise_auth_app(auth_client: AuthClient, atoms_client: AtomsAPIClient):
    auth_app = typer.Typer(name="auth")

    @auth_app.command()
    def login():
        asyncio.run(async_login())

    async def async_login():
        console.print("[bold cyan]Smallest API Key[/bold cyan]")
        console.print(
            "[dim]Enter your Smallest API key from https://console.smallest.ai/apikeys[/dim]"
        )
        api_key = Prompt.ask("> ", password=True)

        try:
            account_details = await atoms_client.get_account_details(api_key)
        except Exception:
            console.print("[red]Invalid API key[/red]")
            return

        auth_client.login(api_key)
        console.print(
            f"[bold green]Login successful [bold green]{account_details.userEmail}[/bold green]"
        )

    @auth_app.command()
    def logout():
        auth_client.logout()
        console.print("[bold green]Logout successful[/bold green]")

    return auth_app



import asyncio
import base64
from io import BytesIO
from pathlib import Path

import questionary
import typer
from rich.console import Console
from rich.panel import Panel
from rich.table import Table

from smallestai.cli.lib.atoms import AgentBuildStatus, AtomsAPIClient
from smallestai.cli.lib.auth import AuthClient
from smallestai.cli.lib.chat import ChatClient, chat_loop
from smallestai.cli.lib.project_config import ProjectConfig
from smallestai.cli.utils import create_zip_from_directory

AGENT_BUILD_STATUS_COLORS = {
    AgentBuildStatus.SUCCEEDED: "green",
    AgentBuildStatus.BUILD_FAILED: "red",
    AgentBuildStatus.DEPLOY_FAILED: "red",
    AgentBuildStatus.QUEUED: "yellow",
    AgentBuildStatus.BUILDING: "yellow",
    AgentBuildStatus.DEPLOYING: "yellow",
}

console = Console()


def initialise_agent_app(
    project_config: ProjectConfig, auth_client: AuthClient, atoms_client: AtomsAPIClient
):
    app = typer.Typer(name="agent")

    @app.command()
    def init():
        """Initialize the agent configuration in the current directory."""
        asyncio.run(async_init())

    async def async_init():
        """Async implementation of init command."""
        agent_id = project_config.get_agent_id()

        if agent_id:
            console.print(
                f"[green]Agent already initialized with ID: [bold]{agent_id}[/bold][/green]"
            )
            return

        # Check if user is logged in
        credentials = auth_client.get_credentials()
        if not credentials or not credentials.get("access_token"):
            console.print(
                "[red]Error: You must be logged in first. Run 'smallestai auth login'[/red]"
            )
            raise typer.Exit(1)

        access_token = credentials["access_token"]

        console.print("[dim]Fetching agents...[/dim]")
        try:
            agent_data = await atoms_client.get_agents(access_token)
        except Exception as e:
            console.print(f"[red]Error fetching agents: {e}[/red]")
            return

        if not agent_data.agents:
            console.print(
                "[yellow]No agents found. Create an agent first at https://console.smallest.ai[/yellow]"
            )
            return

        choices = [
            questionary.Choice(
                title=f"{agent.name} - ID: {agent.id}",
                value=agent.id,
            )
            for agent in agent_data.agents
        ]

        selected_agent = await questionary.select(
            message="Select an agent to link",
            choices=choices,
            pointer="> ",
        ).ask_async()

        if not selected_agent:
            console.print("[red]No agent selected[/red]")
            return

        project_config.set_agent_id(selected_agent)
        console.print("[green]Agent initialized successfully![/green]")

    @app.command()
    def deploy(
        entry_point: str = typer.Option(
            "server.py",
            "--entry-point",
            "-e",
            help="Entry point file name (e.g., server.py)",
        ),
    ):
        """
        Deploy an agent to the Atoms platform.

        Packages the agent code directory into a zip file and deploys it to the backend.
        """
        asyncio.run(async_deploy(".", entry_point))

    async def async_deploy(directory: str, entry_point: str):
        """Deploy an agent asynchronously."""
        agent_id = project_config.get_agent_id()

        if not agent_id:
            console.print(
                "[red]Agent not initialized. Run 'smallestai agent init' first.[/red]"
            )
            return

        # Check if user is logged in
        credentials = auth_client.get_credentials()
        if not credentials or not credentials.get("access_token"):
            console.print(
                "[red]Error: You must be logged in first. Run 'smallestai auth login'[/red]"
            )
            raise typer.Exit(1)

        access_token = credentials["access_token"]

        dir_path = Path(directory)
        if not dir_path.exists():
            console.print(f"[red]Error: Directory '{directory}' does not exist.[/red]")
            return

        if not dir_path.is_dir():
            console.print(f"[red]Error: '{directory}' is not a directory.[/red]")
            return

        # Check if entry point file exists
        entry_point_path = dir_path / entry_point
        if not entry_point_path.exists():
            console.print(
                f"[red]Error: Entry point file '{entry_point}' not found in '{directory}'.[/red]"
            )
            return

        console.print(
            f"[bold cyan]Deploying agent from: {dir_path.absolute()}[/bold cyan]"
        )
        console.print(f"[dim]Entry point: {entry_point}[/dim]")
        console.print(f"[dim]Agent ID: {agent_id}[/dim]\n")

        # Create zip file in memory
        console.print("[yellow]Packaging agent code...[/yellow]")
        zip_buffer: BytesIO = create_zip_from_directory(dir_path)

        zip_base64 = base64.b64encode(zip_buffer.getvalue()).decode("utf-8")
        zip_size_mb = len(zip_buffer.getvalue()) / (1024 * 1024)
        console.print(f"[green]✓[/green] Package created ({zip_size_mb:.2f} MB)")

        # Deploy to backend
        console.print("[yellow]Uploading to Atoms platform...[/yellow]")
        try:
            result = await atoms_client.create_agent_build(
                agent_id=agent_id,
                entry_point_file_name=entry_point,
                agent_code_zip=zip_base64,
                api_key=access_token,
            )

            console.print("[bold green]✓ Deployment successful![/bold green]")
            console.print(f"[dim]Build ID: {result.build_id}[/dim]")
            console.print(f"[dim]Status: {result.message}[/dim]")

        except Exception as e:
            console.print(f"[red]Error deploying agent {e}[/red]")

    @app.command()
    def chat():
        """
        Start a chat session with an Atoms SDK agent.

        Connect to a WebSocket server running an Atoms SDK agent and chat interactively.
        """
        host = "localhost"
        port = 8080
        ws_url = f"ws://{host}:{port}/ws"

        console.print(
            Panel(
                f"[bold]SmallestAI agent Chat Client[/bold]\n\nConnecting to: [cyan]{ws_url}[/cyan]",
                border_style="blue",
            )
        )

        async def run():
            client = ChatClient(ws_url)

            if await client._connect():
                try:
                    await chat_loop(client)
                finally:
                    await client._disconnect()
            else:
                console.print("[red]Failed to connect to agent[/red]")

        asyncio.run(run())

    @app.command("builds")
    def list_builds(
        build_id: str = typer.Argument(
            None, help="Optional build ID to manage directly"
        ),
        limit: int = typer.Option(
            50, "--limit", "-l", help="Number of builds to fetch"
        ),
        offset: int = typer.Option(0, "--offset", "-o", help="Offset for pagination"),
    ):
        """
        List all builds for the current agent and manage them interactively.

        If a build_id is provided, directly manage that specific build.
        """
        asyncio.run(async_list_builds(build_id, limit, offset))

    async def async_list_builds(build_id: str | None, limit: int, offset: int):
        """Async implementation of list builds command."""
        agent_id = project_config.get_agent_id()

        if not agent_id:
            console.print(
                "[red]Agent not initialized. Run 'smallestai agent init' first.[/red]"
            )
            return

        credentials = auth_client.get_credentials()
        if not credentials or not credentials.get("access_token"):
            console.print(
                "[red]Error: You must be logged in first. Run 'smallestai auth login'[/red]"
            )
            raise typer.Exit(1)

        access_token = credentials["access_token"]

        try:
            if build_id:
                console.print(f"[dim]Fetching build: {build_id}[/dim]")
                build = await atoms_client.get_agent_build(
                    agent_id=agent_id,
                    build_id=build_id,
                    api_key=access_token,
                )
                await _manage_build(agent_id, build, access_token)
                return

            console.print(f"[dim]Fetching builds for agent: {agent_id}[/dim]")

            result = await atoms_client.list_agent_builds(
                agent_id=agent_id,
                api_key=access_token,
                limit=limit,
                offset=offset,
            )

            if not result.builds:
                console.print("[yellow]No builds found for this agent.[/yellow]")
                return

            table = Table(title="Agent Builds")
            table.add_column("Build ID", style="cyan")
            table.add_column("Status", style="magenta")
            table.add_column("Live", style="green")
            table.add_column("Created At", style="dim")

            for build in result.builds:
                status_color = AGENT_BUILD_STATUS_COLORS[build.status]
                status_text = (
                    f"[{status_color}]{build.status.value.upper()}[/{status_color}]"
                )

                live_indicator = "[green]✓ LIVE[/green]" if build.is_live else "-"

                table.add_row(
                    build.id,
                    status_text,
                    live_indicator,
                    build.created_at,
                )

            console.print(table)
            console.print(
                f"[dim]Showing {len(result.builds)} of {result.pagination.total} builds[/dim]\n"
            )

            choices = [
                questionary.Choice(
                    title=f"{build.id[:12]}... | {build.status.value} | {'LIVE' if build.is_live else '-'} | {build.created_at}",
                    value=build,
                )
                for build in result.builds
            ]
            choices.append(questionary.Choice(title="Cancel", value="cancel"))

            selected_build = await questionary.select(
                message="Select a build to manage (or Cancel to exit)",
                choices=choices,
                pointer="> ",
            ).ask_async()

            if not selected_build:
                console.print("[dim]No build selected.[/dim]")
                return

            if selected_build == "cancel":
                console.print("[dim]Exiting...[/dim]")
                return

            await _manage_build(agent_id, selected_build, access_token)

        except Exception as e:
            console.print(f"[red]Error: {e}[/red]")

    async def _manage_build(agent_id: str, build, access_token: str):
        """Show action menu and manage a specific build."""
        status_color = AGENT_BUILD_STATUS_COLORS[build.status]
        status_text = f"[{status_color}]{build.status.value.upper()}[/{status_color}]"
        is_live = getattr(build, "is_live", None)
        live_text = "[green]✓ LIVE[/green]" if is_live else "-"

        console.print(
            Panel(
                f"[bold]Build ID:[/bold] {build.id}\n"
                f"[bold]Agent ID:[/bold] {build.agent_id}\n"
                f"[bold]Status:[/bold] {status_text}\n"
                f"[bold]Live:[/bold] {live_text}\n"
                f"[bold]Error Message:[/bold] {getattr(build, 'error_message', None) or '-'}\n"
                f"[bold]Created At:[/bold] {build.created_at}\n"
                f"[bold]Updated At:[/bold] {build.updated_at}",
                title="Build Details",
                border_style="blue",
            )
        )

        if build.status == AgentBuildStatus.SUCCEEDED:
            if is_live:
                action_choices = [
                    questionary.Choice(title="Take Down", value="take_down"),
                    questionary.Choice(title="Cancel", value=None),
                ]
            else:
                action_choices = [
                    questionary.Choice(title="Make Live", value="live"),
                    questionary.Choice(title="Cancel", value=None),
                ]

            selected_action = await questionary.select(
                message=f"What would you like to do with build {build.id[:12]}...?",
                choices=action_choices,
                pointer="> ",
            ).ask_async()

            if not selected_action:
                console.print("[dim]No action selected.[/dim]")
                return

            if selected_action == "live":
                console.print("[yellow]Setting build as live...[/yellow]")
                await atoms_client.update_agent_build(
                    agent_id=agent_id,
                    build_id=build.id,
                    api_key=access_token,
                    is_live=True,
                )
                console.print(
                    f"[bold green]✓ Build {build.id[:12]}... is now LIVE![/bold green]"
                )
            elif selected_action == "take_down":
                console.print("[yellow]Taking down build...[/yellow]")
                await atoms_client.update_agent_build(
                    agent_id=agent_id,
                    build_id=build.id,
                    api_key=access_token,
                    is_live=False,
                )
                console.print(
                    f"[bold green]✓ Build {build.id[:12]}... has been taken down.[/bold green]"
                )

    # @app.command("logs")
    # def stream_build(
    #     build_id: str = typer.Argument(..., help="The build ID to stream logs for"),
    # ):
    #     """
    #     Stream build logs in real-time using Server-Sent Events.
    #     """
    #     asyncio.run(async_stream_build(build_id))

    # async def async_stream_build(build_id: str):
    #     """Async implementation of stream build command."""
    #     agent_id = project_config.get_agent_id()

    #     if not agent_id:
    #         console.print(
    #             "[red]Agent not initialized. Run 'smallestai agent init' first.[/red]"
    #         )
    #         return

    #     credentials = auth_client.get_credentials()
    #     if not credentials or not credentials.get("access_token"):
    #         console.print(
    #             "[red]Error: You must be logged in first. Run 'smallestai auth login'[/red]"
    #         )
    #         raise typer.Exit(1)

    #     access_token = credentials["access_token"]

    #     console.print(f"[bold cyan]Streaming logs for build: {build_id}[/bold cyan]")
    #     console.print("[dim]Press Ctrl+C to stop streaming[/dim]\n")

    #     try:
    #         async for event in atoms_client.stream_agent_build(
    #             agent_id=agent_id,
    #             build_id=build_id,
    #             api_key=access_token,
    #         ):
    #             event_type = event.get("type")

    #             if event_type == "log":
    #                 console.print(f"[dim]LOG:[/dim] {event.get('message', '')}")
    #             elif event_type == "status":
    #                 status = event.get("status", "")
    #                 status_style = {
    #                     "SUCCEEDED": "[green]SUCCEEDED[/green]",
    #                     "BUILD_FAILED": "[red]BUILD_FAILED[/red]",
    #                     "DEPLOY_FAILED": "[red]DEPLOY_FAILED[/red]",
    #                     "PENDING": "[yellow]PENDING[/yellow]",
    #                     "BUILDING": "[yellow]BUILDING[/yellow]",
    #                     "DEPLOYING": "[yellow]DEPLOYING[/yellow]",
    #                 }.get(status, status)
    #                 console.print(f"[bold]STATUS:[/bold] {status_style}")

    #                 if status in ["SUCCEEDED", "BUILD_FAILED", "DEPLOY_FAILED"]:
    #                     console.print("\n[bold]Build stream ended.[/bold]")
    #                     break
    #             elif event_type == "error":
    #                 console.print(f"[red]ERROR:[/red] {event.get('message', '')}")
    #                 break

    #     except KeyboardInterrupt:
    #         console.print("\n[yellow]Streaming stopped by user.[/yellow]")
    #     except Exception as e:
    #         console.print(f"[red]Error streaming build logs: {e}[/red]")

    return app



# This file was auto-generated by Fern from our API Definition.

import typing

SinglePromptConfigToolsItemVariablesExtractionSchemaItemType = typing.Union[
    typing.Literal["text", "number", "boolean", "enum"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ...core.api_error import ApiError
from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.http_response import AsyncHttpResponse, HttpResponse
from ...core.jsonable_encoder import jsonable_encoder
from ...core.request_options import RequestOptions
from ...core.unchecked_base_model import construct_type
from ..errors.bad_request_error import BadRequestError
from ..errors.internal_server_error import InternalServerError
from ..errors.unauthorized_error import UnauthorizedError
from .types.get_waves_voices_request_model import GetWavesVoicesRequestModel
from .types.get_waves_voices_response import GetWavesVoicesResponse


class RawVoicesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get_waves_voices(
        self, model: GetWavesVoicesRequestModel, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[GetWavesVoicesResponse]:
        """
        Get voices supported for a given model using the new Waves API.

        Parameters
        ----------
        model : GetWavesVoicesRequestModel
            The model to use for speech synthesis.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetWavesVoicesResponse]
            Voices retrieved successfully.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/v1/{jsonable_encoder(model)}/get_voices",
            base_url=self._client_wrapper.get_environment().waves,
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetWavesVoicesResponse,
                    construct_type(
                        type_=GetWavesVoicesResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawVoicesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get_waves_voices(
        self, model: GetWavesVoicesRequestModel, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[GetWavesVoicesResponse]:
        """
        Get voices supported for a given model using the new Waves API.

        Parameters
        ----------
        model : GetWavesVoicesRequestModel
            The model to use for speech synthesis.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetWavesVoicesResponse]
            Voices retrieved successfully.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/v1/{jsonable_encoder(model)}/get_voices",
            base_url=self._client_wrapper.get_environment().waves,
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetWavesVoicesResponse,
                    construct_type(
                        type_=GetWavesVoicesResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# coding: utf-8

"""
    Agent Management API

    API for managing agents, their templates, and call logs

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictStr, field_validator
from typing import Any, ClassVar, Dict, List, Optional
from typing import Optional, Set
from typing_extensions import Self

class WebhookSubscription(BaseModel):
    """
    WebhookSubscription
    """ # noqa: E501
    id: Optional[StrictStr] = Field(default=None, description="The unique identifier for the subscription", alias="_id")
    webhook_id: Optional[StrictStr] = Field(default=None, description="The ID of the webhook", alias="webhookId")
    agent_id: Optional[StrictStr] = Field(default=None, description="The ID of the agent", alias="agentId")
    event_type: Optional[StrictStr] = Field(default=None, description="The type of event subscribed to", alias="eventType")
    created_at: Optional[datetime] = Field(default=None, description="The date and time when the subscription was created", alias="createdAt")
    updated_at: Optional[datetime] = Field(default=None, description="The date and time when the subscription was last updated", alias="updatedAt")
    __properties: ClassVar[List[str]] = ["_id", "webhookId", "agentId", "eventType", "createdAt", "updatedAt"]

    @field_validator('event_type')
    def event_type_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in set(['pre-conversation', 'post-conversation']):
            raise ValueError("must be one of enum values ('pre-conversation', 'post-conversation')")
        return value

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of WebhookSubscription from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of WebhookSubscription from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "_id": obj.get("_id"),
            "webhookId": obj.get("webhookId"),
            "agentId": obj.get("agentId"),
            "eventType": obj.get("eventType"),
            "createdAt": obj.get("createdAt"),
            "updatedAt": obj.get("updatedAt")
        })
        return _obj


